// appropriate client for our communication server
// communication server (c) mg sai uni ulm january 2003
// used commands for server:
// QUIT!		... I am leaving now ...
// SEND!user!text	send text "text" to user "user"

#include <qsocket.h>
#include <qapplication.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlistbox.h>
#include <qtextedit.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtextstream.h>
#include <unistd.h>
#include <iostream>

#define PORT 33333		// out standard port 

// QVbox: all added widgets will be displayed vertically in their insert order 
class Client : public QVBox
{
    Q_OBJECT			// provided for own slots
public:
    Client( const QString &host, int port )
    {
	errorState = -1;	// null value for network error state
        // GUI layout
	resize(600, 400);
	setSpacing(5);		// distance in pixels between widgets
	setPaletteBackgroundColor( QColor( 255, 170, 0 ) ); // I like orange ...

	// set up text box
        QHBox *hb0 = new QHBox( this );
        infoText = new QTextEdit( hb0 );// display connect status & server data
	infoText->setReadOnly(true);	// only for displaying text ...
	infoText->setTextFormat(Qt::PlainText); // don't interpret HTML-Tags!

	// listbox for online users
	userlist = new QListBox(hb0);

	// set up user name field
        QHBox *hb1 = new QHBox( this );
	QLabel *txt1 = new QLabel("username: ", hb1);
        username = new QLineEdit( hb1 );// text input box

	// set up buttons and text field
        QHBox *hb2 = new QHBox( this );
	QLabel *txt2 = new QLabel("enter text: ", hb2);
        inputText = new QLineEdit( hb2 );// text input box
	inputText->setReadOnly(true);	// wait for username!
	// quit button
        QPushButton *quit = new QPushButton( "quit" , this );

	// send username to server when return was pressed
        connect( username, SIGNAL(returnPressed()), SLOT(sendName()) );
	// send textbox data to server when return was pressed
        connect( inputText, SIGNAL(returnPressed()), SLOT(sendToServer()) );
	// close connection and quit application
        connect( quit, SIGNAL(clicked()), this, SLOT(closeConnection()) );

        // create socket and connect various of its signals
        socket = new QSocket( this );	// "empty" socket - not yet connected
	// connection to server established
        connect( socket, SIGNAL(connected()), SLOT(socketConnected()) );
	// connection to server closed
        connect( socket, SIGNAL(connectionClosed()),
			SLOT(socketConnectionClosed()) );
	// data from server is arriving
        connect( socket, SIGNAL(readyRead()), SLOT(socketReadyRead()) );
	// an communication error occured
        connect( socket, SIGNAL(error(int)), SLOT(socketError(int)) );

        infoText->insert( "Trying to connect to server\n" );
        // connect to server if possible - connection calls then socketConnected
        socket->connectToHost( host, port );
    }

    ~Client() { }

private slots:
    void closeConnection() {	// used for closing the socket connection
	QTextStream os(socket);
	os<<"QUIT!"<<username->text()<<"\n";;
	connect( socket, SIGNAL(delayedCloseFinished()), qApp, SLOT(quit()) );
        socket->close();	// shutdown socket connection
    }

    void sendName() {		// send username to server and enable textfield
	if (!username->text().isEmpty()) {
		QTextStream os(socket);
		os << "USER!"<<username->text() << "\n";
		username->setReadOnly(true);
		inputText->setReadOnly(false);
	}
    }

    void sendToServer() {	// used to send one text line to the server
	if (!inputText->text().isEmpty()) {
		QTextStream os(socket);
		// send line to server
		os <<"SEND!"<<userlist->currentText()<<"!";
		os << inputText->text() << "\n";
		// display text line in textedit field
		infoText->insert(inputText->text()+"\n");
		inputText->setText("");	// clear input line
	}
    }

    void socketReadyRead() {	// slot called when server data arrives
		// wait for a complete line (terminated by newline)
        while (socket->canReadLine()) {
	    QString text = socket->readLine();
	    text = text.left(text.length()-1);      	// delete cr
	    if (text.contains("\r")>0)      		// linefeed?
		    text = text.left(text.length()-1);      // delete lf too
	    if (text.contains("LIST:")>0) {	// user list is arriving
		QString oldbuddy = userlist->currentText();
		text = text.right(text.length()-5);
		int pos = text.find(':');		// look for ':'
		userlist->clear();			// clear old user list
		while (pos>0) {				// enter new list
			userlist->insertItem(text.left(pos));
			text = text.right(text.length()-pos-1);
			pos = text.find(':');
		}
		// find name of old communication partner
		QListBoxItem *old = userlist->findItem(oldbuddy);
		if (old) // set old communication partner active in new list
			userlist->setCurrentItem(old);
	    } else 	 // no command - then write incoming line to text box
		infoText->insert("> "+text+"\n");
	}
    }

    void socketConnected() {	// called, when server connection is established
        infoText->insert( "Connection to server established!\n" );
	infoText->insert("Enter now your username!\n");
    }

    void socketConnectionClosed() { // server closes the connection !!
        infoText->insert( "Connection closed by the server\n" );
    }

    // network error ??
    void socketError( int e ) {	// arg() substitutes %1 in QString!
	errorState = e;
        infoText->insert( QString("Error number %1 occurred\n").arg(e) );
	if (e==QSocket::ErrConnectionRefused) {
		infoText->insert( "no server available??? \n");
		infoText->insert( "leaving now ...\n");
		std::cerr <<"connection to server failed ..."<<std::endl;
		qApp->quit();
	}
    }

private:
    QTextEdit *infoText;	// displayed chat text
    QLineEdit *username;	// my user name
    QLineEdit *inputText;	// input text line for chat
    QListBox *userlist;		// current list of participating users
    QSocket *socket;		// socket connection to server
public:
    int errorState;		// determine error state!
};


int main( int argc, char** argv )
{
    QApplication app( argc, argv );
    // connect to local host if argv[1]not specified
    Client client( argc<2 ? "localhost" : argv[1], PORT );
    if (client.errorState==QSocket::ErrConnectionRefused) {
	std::cerr <<"no connection to server ";
	std::cerr <<(argc<2 ? "localhost" : argv[1]);
	std::cerr <<" on port "<<PORT<<std::endl;
	return 1;		// no connection to server ...
    } else {
	app.setMainWidget( &client );
	client.show();
	return app.exec();
    }
	
}
#include "client.moc"		// include instead of link moc-file
