/*
 * A simple mirroring HTTP server in Qt.
 *
 * Copyright 2001, Matthias Kalle Dalheimer
 */

#include <qapplication.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qsocket.h>
#include <qserversocket.h>
#include <qnetwork.h>
#include "httpserv.h"
#include <time.h>

// liefert aktuelle Uhrzeit als char* zurck
static char *myTime() {
	const time_t now = time((time_t*)0);

	return ctime(&now);
}



void HTTPMirrorServer::newConnection( int socketId )
{
  QSocket* socket = new QSocket( this );
  socket->setSocket( socketId );
  connect( socket, SIGNAL( readyRead() ),
		   this, SLOT( slotIncomingData() ) );
  connect( socket, SIGNAL( delayedCloseFinished() ),
		   this, SLOT( slotDeleteSocket() ) );

  // Send the HTML header to the socket
  QTextStream stream( socket );
  stream << "<HTML><BODY><PRE>";

}


/* This slot is called when data arrives via one of the data
   sockets. It reads the header until an empty line occurs, and
   echoes it back to the client. */
void HTTPMirrorServer::slotIncomingData()
{
  // Retrieve the QSocket object that corresponds to the handle. There
  // could be more than one, so we need to find out who sent the signal.
  QSocket* socket = (QSocket*) sender();

  // Check whether a full line of data is available; if not, do not
  // read the data now, but wait for the next data to come in. If yes,
  // read the data line by line, and echo it back until a blank line
  // (which marks the end of the header) occurs.
  QTextStream stream( socket );

  while( socket->canReadLine() ) {
	QString line = socket->readLine();
	if( line == "\r\n" ) {
	  // last line was blank, header is finished

	  // send the HTML end
	  QHostAddress add = socket->peerAddress();
	  if (ok()) {
		  stream <<"------"<<endl;
		  stream << "ClientIP = "<<add.toString()<<endl;
		  stream << "Time: = "<<myTime()<<endl;
	 }
	  else
		stream <<"Socket not ok???"<<endl;
	  stream << "</PRE></BODY></HTML>";

	  // close the socket. This will make the socket emit
	  // delayedCloseFinished() which calls slotDeleteSocket() which
	  // deletes the socket.
	  socket->close();
	  break;
	} else {
	  // echo the line back to the client
	  stream << line;

	}
        // we'll get back in here as soon as the next line is available
  }
}


/* This slot is called after the socket has been closed. */
void HTTPMirrorServer::slotDeleteSocket()
{
  // There could be more than one socket, so we need to find out who
  // sent the signal.
  QSocket* socket = (QSocket*) sender();
  // Looks dangerous, but is OK.
  delete socket;
}

int main( int argc, char* argv[] )
{
  // We need a QApplication object, but there won't be any widgets.
  QApplication a( argc, argv, false );

  // Creating a server object starts the server.
  HTTPMirrorServer server;

  return a.exec();
}
